/**
* This module contains vanilla javascript Event listeners which are listening
* for button clicks etc. the user is doing to interact with the page.
*/

// Import the script that implements a spinner animation for buttons.
import {
  loadingSpinnerHTML,
} from './spinner.js';

/**
 * The following listener handles what functions are called when the user
 * does use the page navigation to navigate to a new page.
 */
function pageNavigation(resultsList, results, client) {
  for (let element of resultsList.pagination) {
    element.addEventListener("click", (event) => {
      // Shows match context according to the user picked value on a new page.
      resultsList.changeContext();
      // De- or activates expertMode on new page depending on switch value.
      if (resultsList.displayOptionsFormExpertMode.checked) {
        resultsList.expertModeOn('query-display', results);
      } else {
        resultsList.expertModeOff('query-display');
      }
      // Activates inspect buttons on new page if client is not busy.
      resultsList.toggleInspectButtons(client);
    });
  }
}

/**
 * The following event Listener handles the expert mode switch for the list.
 */
function expertModeSwitch(resultsList, results) {
  resultsList.displayOptionsFormExpertMode.onchange = (event) => {
    if (event.target.checked) {
      resultsList.expertModeOn('query-display', results);
    } else {
      resultsList.expertModeOff('query-display');
    }
  };
}

/**
 * The following event Listener handles the add-btn and the inspect-btn
 * onclick events via bubbleing.
 */
function actionButtons(resultsList, results, client) {
  resultsList.queryResultsTable.addEventListener('click', (event) => {
    let dataIndex;
    if (event.target.classList.contains('inspect-btn')) {
      dataIndex = parseInt(event.target.closest('tr').dataset.index);
      resultsList.inspect(client, results, [dataIndex], 'inspect');
    } else if (event.target.classList.contains('add-btn')) {
      dataIndex = parseInt(event.target.closest('tr').dataset.index);
      resultsList.addToSubResults(dataIndex, client);
    }
  })
}

/**
 * Following event listeners handle the change of Context size per match and
 * the number of matches shown per page.
 */
function displayOptions(resultsList, results, client) {
  resultsList.displayOptionsFormResultsPerPage.onchange = (event) => {
    resultsList.changeHitsPerPage(client, results);
  };
  resultsList.displayOptionsFormResultContext.onchange = (event) => {
    resultsList.changeContext();
  };
}

/**
 * The following event listener handles the show metadata button and its
 * functionality.
 */
function showMetaData(resultsList, results) {
  resultsList.showMetaData.onclick = () => {
    resultsList.metaDataModalContent.textContent = '';
    let table = resultsList.createMetaDataForModal(results.metaData);
    resultsList.metaDataModalContent.insertAdjacentHTML('afterbegin', table);
    resultsList.metaDataModal.open();
    let collapsibles = resultsList.metaDataModalContent.querySelectorAll(".text-metadata");
    for (let collapsible of collapsibles) {
      collapsible.onclick = () => {
        let elems = resultsList.metaDataModalContent.querySelectorAll('.collapsible');
        let instances = M.Collapsible.init(elems, {accordion: false});
        resultsList.createTextDetails(results.metaData);
      }
    }
  };
}

/**
 * The following event listener handles the button showing infos about matches
 * and their corresponding corpus files
 */

function showCorpusFiles(resultsList, results) {
  resultsList.showCorpusFiles.onclick = () => {
    resultsList.showCorpusFilesModalContent.innerHTML = '';
    let htmlString = `
    <div id="corpus-file-table">
      <ul class="pagination paginationTop"></ul>
      <table class="responsive-table highlight">
        <thead>
          <tr>
            <th class="sort" data-sort="title">Title</th>
            <th class="sort" data-sort="year">Year</th>
            <th class="sort" data-sort="match-count">Match count in this text</th>
          </tr>
        </thead>
        <tbody class="list">
    `
    for (let [key, value] of Object.entries(results.data.text_lookup)) {
      htmlString += `
        <tr>
          <td class="title">${value.title}</td>
          <td class="year">${value.publishing_year}</td>
          <td class="match-count">${value.match_count}</td>
        </tr>
      `
    }
    htmlString += `
      </tbody>
      </table>
      <ul class="pagination paginationBottom"></ul>
    </div>
      `
    resultsList.showCorpusFilesModalContent.insertAdjacentHTML('afterbegin', htmlString);
    resultsList.showCorpusFilesModal.open();
    let options = {
      page: 10,
      pagination: [{
        name: "paginationTop",
        paginationClass: "paginationTop",
        innerWindow: 8,
        outerWindow: 1
      }, {
        paginationClass: "paginationBottom",
        innerWindow: 8,
        outerWindow: 1
      }],
      valueNames: ["title", "year", "match-count"],
    };
    let corpusFileTable = new List('corpus-file-table', options);
    console.log(corpusFileTable);
  }
}

/**
 * Checks if resultsList.exportFullInspectContext switch is changed.
 * If it has been changed reset all Download buttons.
 */
function exportFullContextSwitch(resultsList) {
  resultsList.exportFullInspectContext.onchange = (event) => {
    // Hide all download buttons.
    resultsList.fullResultsExport.classList.toggle('hide', true);
    resultsList.subResultsExport.classList.toggle('hide', true);
    // Show result create buttons.
    resultsList.fullResultsCreate.classList.toggle('hide', false);
    resultsList.subResultsCreate.classList.toggle('hide', false);
  }
}

/**
 * The following event listeners are handeling the data export.
 * 1. Create full-results
 * 2. Create sub-results
 * 3. Download full-results
 * 4. Download sub-results
 * 5. Download single inspect-results
 */

// 1. Add events for full-results create
function createFullResults(resultsList, results) {
  resultsList.fullResultsCreate.onclick = (event) => {
    resultsList.fullResultsCreate.querySelector('i').classList.toggle('hide');
    resultsList.fullResultsCreate.innerText = 'Creating...';
    resultsList.fullResultsCreate.insertAdjacentHTML('afterbegin',
                                                      loadingSpinnerHTML);
    // .keys() is for a zero based array. I think...
    let dataIndexes = [...Array(results.data.match_count).keys()];
    // Empty fullResultsData so that no previous data is used.
    results.fullResultsData.init();
    resultsList.notifyClient('get-results', {resultsType: 'full-results',
                                             dataIndexes: dataIndexes,
                                             resultsList: resultsList,});
  }
}

// 2. Add events for sub-results create
function createSubResults(resultsList, results) {
  resultsList.subResultsCreate.onclick = (event) => {
    let dataIndexes = [];
    Object.keys(resultsList.subResultsIndexes).forEach((id) => {
      dataIndexes.push(id);
    });
    resultsList.subResultsCreate.querySelector('i').classList.toggle('hide');
    resultsList.subResultsCreate.innerText = 'Creating...';
    resultsList.subResultsCreate.insertAdjacentHTML('afterbegin',
                                                     loadingSpinnerHTML);
    // Empty subResultsData so that no previous data is used.
    results.subResultsData.init();
    resultsList.notifyClient('get-results', {resultsType: 'sub-results',
                                             dataIndexes: dataIndexes,
                                             resultsList: resultsList,});
  }
}
// 3. Open download modal when full results export button is pressed
function exportFullResults(resultsList, results) {
  resultsList.fullResultsExport.onclick = (event) => {
    resultsList.queryResultsDownloadModal.open();
    // add onclick to download JSON button and download the file
    resultsList.downloadResultsJson.onclick = (event) => {
      let suffix = 'full-results'
      if (resultsList.exportFullInspectContext.checked) {
        suffix += '_full-context';
      }
      let filename = results.fullResultsData.createDownloadFilename(suffix);
      results.fullResultsData.addData(results.metaData);
      results.fullResultsData.downloadJSONRessource(filename,
                                                    results.fullResultsData,
                                                    resultsList.downloadResultsJson)};
  }
}
// 4. Open download modal when sub results export button is pressed
function exportSubResults(resultsList, results) {
  resultsList.subResultsExport.onclick = (event) => {
    resultsList.queryResultsDownloadModal.open();
    // add onclick to download JSON button and download the file
    resultsList.downloadResultsJson.onclick = (event) => {
      let suffix = 'sub-results'
      if (resultsList.exportFullInspectContext.checked) {
        suffix += '_full-context';
      }
      let filename = results.subResultsData.createDownloadFilename(suffix);
      results.subResultsData.addData(results.metaData);
      results.subResultsData.downloadJSONRessource(filename,
                                                   results.subResultsData,
                                                   resultsList.downloadResultsJson)};
  }
}
// 5. Open download modal when inspect-results-export button is pressed
function exportSingleMatch(resultsList, results) {
  resultsList.inspectResultsExport.onclick = (event) => {
    resultsList.queryResultsDownloadModal.open();
    // add onclick to download JSON button and download the file
    resultsList.downloadResultsJson.onclick = (event) => {
      let filename = results.subResultsData.createDownloadFilename('inspect-results_full-context');
      results.subResultsData.addData(results.metaData);
      results.subResultsData.downloadJSONRessource(filename,
                                                   results.inspectResultsData,
                                                   resultsList.downloadResultsJson)};
  }
}

export {
  pageNavigation,
  expertModeSwitch,
  actionButtons,
  displayOptions,
  showMetaData,
  showCorpusFiles,
  exportFullContextSwitch,
  createFullResults,
  createSubResults,
  exportFullResults,
  exportSubResults,
  exportSingleMatch,
}
